package cz.zcu.kiv;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import org.xml.sax.InputSource;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFWriter;
import com.hp.hpl.jena.shared.BadURIException;
/**
 * This class provides all necessary methods to work with In/Out PutStreams.
 * @author Hain
 *
 */
class Files {
	/**
	 * transform input stream to input source.
	 * @param in Input stream.
	 * @param coding File encoding.
	 * @return The loaded file.
	 * @throws FileNotFoundException File not found.
	 * @throws UnsupportedEncodingException	unsupported encoding.
	 */
	@SuppressWarnings("static-method")
	InputSource openFile(InputStream in, String coding) throws FileNotFoundException, UnsupportedEncodingException{
		BufferedReader input = new BufferedReader(new InputStreamReader(in, coding));
		InputSource file = new InputSource(input);
		file.setEncoding(coding);
		return file;
	}
	
	/**
	 * Write solution to output stream.
	 * @param outputFile Output stream.
	 * @param model RDF model.
	 * @param outputFormat Output format.
	 * @throws BadURIException Bad URI.
	 * @throws FileNotFoundException File not found.
	 * @throws UnsupportedEncodingException	Unsupported encoding.
	 * @throws IOException Error saving.
	 */
	@SuppressWarnings("static-method")
	void writeToFile(OutputStream outputFile, Model model, String outputFormat) throws BadURIException, FileNotFoundException, UnsupportedEncodingException, IOException{
		BufferedWriter output = new BufferedWriter(new OutputStreamWriter(outputFile, "UTF-8"));
		RDFWriter writer = model.getWriter(outputFormat);
		writer.write(model, output, outputFormat);
		output.close();
	}
}