package cz.zcu.kiv;

import java.util.LinkedList;
import java.util.ListIterator;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

/**
 * This class parsing XML files by SAX and making elements by structure of given file. It is
 * working closely with classes URI, Element and Tag. New Elements are stored in linked list,
 * witch is given to class Jena. 
 * @author Masopust
 *
 */
class RDFmaker extends DefaultHandler { 

	/**
	 * A list containing the currently processed elements and their URIs.
	 */
	private LinkedList<Tag> list = new LinkedList<Tag>();
	
	/**
	 * Output list of Elements, which uses Jena to create the model.
	 */
	private static LinkedList<Element> input = new LinkedList<Element>();
	
	/**
	 * Auxiliary Tag.
	 */
	private Tag temp = new Tag(null, null);
	
	/**
	 * Position indicator in the file.
	 */
	private char iterator = 's';
	
	/**
	 * Keeps the information about whether it was in the previous tag some content.
	 */
	private boolean wasContent = false;
	
	/**
	 * Contains unique URI beginning. Default value: http://
	 */
	private static String prefix;
	
	/**
	 * Auxiliary String for storing the current URI.
	 */
	private String URI;
	
	/**
	 * The auxiliary element to scroll through the list.
	 */
	private URI tmp;
	
	/**
	 * List iterator.
	 */
	private ListIterator<URI> listIterator;
	
	/**
	 * Auxiliary list.
	 */
	private LinkedList<URI> tmpList = new LinkedList<URI>();
	
	/**
	 * Determines whether the current element can be processed. 
	 * Processing may be only when oldTmp is <= 1
	 */
	private int oldTmp;
	
	/**
	 * Determines whether the attributes have been processed.
	 */
	private boolean processedAttribute = false;
	
	/**
	 * Determines the position within the tag.
	 */
	private boolean startB = false;
	
	/**
	 * Determines the position within the tag.
	 */
	private boolean endB = false;

/**
 * This initializes SAX parser and set his parameters to ignore any DTD.
 *  
 * @param in Input file.
 * @param prefixN URI prefix.
 * @return LinkedList<Element> List of elements.
 */
	LinkedList<Element> XML(InputSource in, String prefixN) throws Exception{
		prefix = prefixN;
		SAXParserFactory factory = SAXParserFactory.newInstance(); 
		factory.setValidating(false);
		
		factory.setFeature("http://xml.org/sax/features/validation", false);
		factory.setFeature("http://apache.org/xml/features/nonvalidating/load-dtd-grammar", false);
		factory.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
		factory.setFeature("http://xml.org/sax/features/external-general-entities", false);
		factory.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
		factory.setFeature("http://xml.org/sax/features/use-entity-resolver2", false);	
		factory.setFeature("http://apache.org/xml/features/validation/unparsed-entity-checking", false);
		factory.setFeature("http://xml.org/sax/features/resolve-dtd-uris", false);
		factory.setFeature("http://xml.org/sax/features/resolve-dtd-uris", false);
		factory.setFeature("http://apache.org/xml/features/validation/dynamic", false);
		factory.setFeature("http://apache.org/xml/features/validation/schema/augment-psvi", false);

		SAXParser parser = factory.newSAXParser();
		XMLReader reader = parser.getXMLReader(); 
			
		RDFmaker test = new RDFmaker(); 
		reader.setContentHandler(test); 
		reader.setErrorHandler(test); 
			
		reader.setFeature("http://xml.org/sax/features/validation", false);
		reader.setFeature("http://apache.org/xml/features/nonvalidating/load-dtd-grammar", false);
		reader.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
		reader.setFeature("http://xml.org/sax/features/external-general-entities", false);
		reader.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
		reader.setFeature("http://xml.org/sax/features/use-entity-resolver2", false);	
		reader.setFeature("http://apache.org/xml/features/validation/unparsed-entity-checking", false);
		reader.setFeature("http://xml.org/sax/features/resolve-dtd-uris", false);
		reader.setFeature("http://xml.org/sax/features/resolve-dtd-uris", false);
		reader.setFeature("http://apache.org/xml/features/validation/dynamic", false);
		reader.setFeature("http://apache.org/xml/features/validation/schema/augment-psvi", false);
		
		reader.parse(in);
		return input;
	} 

	@Override
	public void startDocument() throws SAXException {/** all possible SAX exceptions*/ } 

	/**
	 * If this method is called, you are in the opening tag. 
	 * Create an auxiliary tag here. Then set oldTmp to 0.
	 * 
	 * @param uri URI
	 * @param localName Tag name in case control namespaces.
	 * @param qName Tag name.
	 * @param attributes List of attributes.
	 */
	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		this.wasContent = false;
		this.processedAttribute = false;
		this.startB = true;
		this.iterator = 's';
		this.temp = new Tag(null, null);
		this.temp.setName(qName);
		this.temp.setAttributes(attributes);
		this.list.addFirst(new Tag(this.temp));
		this.oldTmp = 0;

	} 
	
	/**
	 * If your document contains a DTD, 
	 * then the scheme is likely to change reading. This method ensures call to characters ().
	 * 
	 * @param ch Char array forming the Tag.
	 * @param start Start index.
	 * @param length Array length.
	 */
	@Override
	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
		 characters(ch, 0, 0);
   }

	/**
	 * Tag set to the position 'e'. Increase the value of oldTmp by 1.
	 */
	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException {
		//System.out.println("End");
		this.iterator = 'e';
		this.endB = true;
		this.list.removeFirst();
		this.oldTmp++;
	} 

	/**
	 * This method is Processing Elements. 
	 * It determines if the Element is continuous or final. 
	 * It also adding unique URI to continual elements.
	 * 
	 * @param ch Char array forming the Tag.
	 * @param start Start index.
	 * @param length Array length.
	 */
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException  {
		//System.out.println("Char");
		String sText = new String(ch, start, length);
		sText = sText.trim();
		if(sText.length() >  0){									
			this.wasContent = true;
			if(this.temp.getAttributes().getLength() > 0){
				this.generateURI();
				if(this.list.size() > 1){
					createMiddleElementRDF();
				} 
				checkAtributStart();
				createEndElementRDFToTagWithAtribut(this.temp.getName(), sText);
				//vytvorKonecnyPrvekRDF(temp.getJmeno(), sText);
			}
			else{
				createEndElementRDF(this.temp.getName(), sText);
			}
		}
		else if (this.wasContent == false){
			switch(this.iterator){

			case 's': if(this.startB){
				      		this.generateURI();
				      		if(this.list.size() > 1){
				      			createMiddleElementRDF();
				      			if(this.list.getFirst().getAttributes().getLength() > 0){
					      			checkAtributes();
					      			this.processedAttribute = true;
					      		}
				      		} else{
				      			if(this.list.getFirst().getAttributes().getLength() > 0){
					      			checkAtributStart();
					      			this.processedAttribute = true;
					      		}
				      		}
				      		this.startB = false;
					  }
					  break;
					  
			case 'e': if(this.endB){
					  	  if(this.temp.getAttributes().getLength() > 0 && this.oldTmp <= 1 && this.processedAttribute == false){
					  		    String identifikator = generateURIAtribut();
					  		    if(this.list.size() >= 1){
					  		    	createMiddleElementRDF(identifikator);
					  		    }
					  		    checkAtributes(identifikator);
					  		    this.processedAttribute = false;
					  	  }
					  	  this.endB = false;
					  }
					  break;
			}
		}
	} 

	
	/**
	 * This is making new end element.
	 * 
	 * @param predicate vazba prvku
	 * @param object vazany prvek
	 */
	private void createEndElementRDFToTagWithAtribut(String predicate, String object) {
		Element novy = new Element(this.list.getFirst().getOwnURI(), predicate, object, false);
		input.add(novy);
	}

	/**
	 * Checks a list of attributes.
	 */
	private void checkAtributStart() {
		if(this.temp.getAttributes().getLength() > 0){
			for(int i = 0; i < this.temp.getAttributes().getLength(); i++){
				createEndElementRDFBegin(this.temp.getAttributes().getQName(i), this.temp.getAttributes().getValue(i));
			}
		}
	}
	
	/**
	 * Checks a list of attributes.
	 * 
	 * @param identifier URI URI of the element to which attributes are to be administered.
	 */
	private void checkAtributes(String identifier) {
		if(this.temp.getAttributes().getLength() > 0){
			for(int i = 0; i < this.temp.getAttributes().getLength(); i++){
				createEndElementRDFFromAtribte(identifier, this.temp.getAttributes().getQName(i), this.temp.getAttributes().getValue(i));
			}
		}
	}
	
	/**
	 * Checks a list of attributes.
	 */
	private void checkAtributes() {
		if(this.temp.getAttributes().getLength() > 0){
			for(int i = 0; i < this.temp.getAttributes().getLength(); i++){
				createEndElementRDF(this.temp.getAttributes().getQName(i), this.temp.getAttributes().getValue(i));
			}
		}	
	}
	
	/**
	 * This is making new end Element.
	 * 
	 * @param predicate 
	 * @param object
	 */
	private void createEndElementRDFBegin(String predicate, String object) {
		Element novy = new Element(this.list.getFirst().getOwnURI(), predicate, object, false);
		input.add(novy);
	}


	/**
	 * This is making new middle element.
	 * 
	 * @param identifier Last Element URI. 
	 */
	private void createMiddleElementRDF(String identifier) {
		Element tmpE = new Element(this.list.getFirst().getOwnURI(), "isElementOf", identifier, true);
		input.add(tmpE);	
	}

	/**
	 * Methods for generating a URI to end elements.
	 * 
	 * @return URI
	 */
	private String generateURIAtribut() {
		this.tmpList = this.list.getFirst().getListURIsChildren();
		this.listIterator = this.tmpList.listIterator();
		while(this.listIterator.hasNext()){
			this.tmp = this.listIterator.next();
			if(this.tmp.getName() == this.temp.getName()){
				this.URI = this.list.getFirst().getOwnURI() + "/" + this.temp.getName() + "" + this.tmp.getCounter();
				this.tmp.setCounter(this.tmp.getCounter() + 1);
				return this.URI;
			}
		}
		this.list.getFirst().getListURIsChildren().add(new URI(this.temp.getName()));
		this.URI = this.list.getFirst().getOwnURI() + "/" + this.temp.getName(); 
		return this.URI;	
	}
	/**
	 * This is making new end element.
	 * @param identifier Last element URI.
	 * @param predicate Predicate.
	 * @param object Object.
	 */
	@SuppressWarnings("static-method")
	private void createEndElementRDFFromAtribte(String identifier, String predicate, String object) {
		Element newE = new Element(identifier, predicate, object, false);
		input.add(newE);	
	}
	/**
	 * This is making new middle element.
	 */
	private void createMiddleElementRDF() {
		Element newE = new Element(this.list.get(1).getOwnURI(), "isElementOf", this.list.getFirst().getOwnURI(), true);
		input.add(newE);
	}
	/**
	 * This is making new end element.
	 * @param predicate Predicate.
	 * @param object Object.
	 */
	private void createEndElementRDF(String predicate, String object) {
		Element newE = new Element(this.list.get(1).getOwnURI(), predicate, object, false);
		input.add(newE);
	}
	/**
	 * This is generating unique URI to elements.
	 */
	private void generateURI() {
		if(this.list.size() == 1){
			this.URI = prefix + this.temp.getName();
			this.list.getFirst().setOwnURI(this.URI);
		} else{
			this.list.getFirst().setOwnURI(findURIforTemp());
		}
	}
	/**
	 * The method tries to find an auxiliary tag URI.
	 * @return URI
	 */
	private String findURIforTemp() {
		this.tmpList = this.list.get(1).getListURIsChildren();
		this.listIterator = this.tmpList.listIterator();
		while(this.listIterator.hasNext()){
			this.tmp = this.listIterator.next();
			if(this.tmp.getName() == this.temp.getName()){
				this.URI = this.list.get(1).getOwnURI() + "/" + this.temp.getName() + "" + this.tmp.getCounter();
				this.tmp.setCounter(this.tmp.getCounter() + 1);
				return this.URI;
			}
		}
		this.list.get(1).getListURIsChildren().add(new URI(this.temp.getName()));
		this.URI = this.list.get(1).getOwnURI() + "/" + this.temp.getName(); 
		return this.URI;
	}

	@Override
	public void endDocument(){/** all possible SAX exceptions*/  } 

	@Override
	public void error(SAXParseException e) {/** all possible SAX exceptions*/  } 

	@Override
	public void warning(SAXParseException e) {/** all possible SAX exceptions*/  } 

	@Override
	public void fatalError(SAXParseException e) {/** all possible SAX exceptions*/  }

} 