package cz.zcu.kiv;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.LinkedList;
import org.xml.sax.InputSource;

import com.hp.hpl.jena.rdf.model.Model;
/**
 * This class provides an interface for using this library.
 * 
 * This library makes from given <b>inPutStream</b> (some valid XML file) 
 * a valid RDF file to <b>outPutStream</b>. 
 * <br><br>
 * Only method, which you can use is <i>transformXMLtoRDF()</i>
 * <br><br>
 * You can specify 5 additional things:
 * <br>	setEncoding
 * <br> setNameSpace
 * <br> setRelation
 * <br> setOutputFormat
 * <br> setPrefix
 * 
 * @see https://sourceforge.net/p/zswi2012rdf/wiki/Home/
 * @author Masopust
 * @version 1.0.1
 * 
 */
public class XMLtoRDF {
	/**
	 * Default output file name.
	 */
	private OutputStream outputFile;
	/**
	 * Default namespace.
	 */
	private String namespace = "eeg";
	/**
	 * Default relation.
	 */
	private String relation = "http://relation/";
	/**
	 * Default encoding.
	 */
	private String encoding = "UTF-8";
	/**
	 * Default output format.
	 */
	private String outputFormat = "RDF/XML";
	/**
	 * Default prefix.
	 */
	private String prefix = "http://";
	
	/**
	 * 
	 * Set the expected encoding of given XML file (<b>inPutStream</b>).
	 * 
	 * @param encodingN like "UTF-8"
	 */
	public void setEncoding(String encodingN){
		this.encoding = encodingN;
	}
	/**
	 * Set the name space.
	 * @param ns like "eeg"
	 */
	public void setNameSpace(String ns){
		this.namespace = ns;
	}
	/**
	 * Set the relation name.
	 * @param rel like "http://relation/"
	 */
	public void setRelation(String rel){
		this.relation = rel;
	}
	/**
	 * Set the <b>outPutStream</b> RDF file format.
	 * <br>Only two usable RDF outPuts are: <br>
	 * <br> "RDF/XML-ABBREV" - Slow, produces readable output.
	 * <br> "RDF/XML"        - Fast, produces regular output , but it is not readable.
	 * 
	 * @see http://jena.apache.org/documentation/io/iohowto.html#quick-introduction
	 * 
	 * @param of Output format like "RDF/XML"
	 */
	public void setOutputFormat(String of){
		this.outputFormat = of;
	}
	/**
	 * Set the preffix.
	 * @param p Prefix like "http://"
	 */
	public void setPrefix(String p){
		this.prefix = p;
	}
	
	/**
	 * The "main" method. When called, it takes the inPutStream (has to be valid XML)
	 * and try to parse it and make valid RDF file (outPutStream).
	 * 
	 * @param is Input stream.
	 * @param os Output stream.
	 * @throws Exception
	 */
	public void transformXMLtoRDF(InputStream is, OutputStream os) throws Exception{
		this.outputFile = os;
		InputSource vstup = new Files().openFile(is, this.encoding);
		LinkedList<Element> seznam = new RDFmaker().XML(vstup, this.prefix);
		Model model = new Jena().createRDF(seznam, this.namespace, this.relation);
		new Files().writeToFile(this.outputFile, model, this.outputFormat);
	}
}

