package cz.zcu.kiv.eegdsp.fourier;


import java.util.Arrays;

import org.apache.commons.math.MathException;
import org.apache.commons.math.complex.Complex;
import org.apache.commons.math.transform.FastFourierTransformer;

import cz.zcu.kiv.eegdsp.common.ISignalProcessor;
import cz.zcu.kiv.eegdsp.utils.Mathematic;

/**
 * Execution interface for Fast Fourier Transformation (FFT).
 * Singleton.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class FastFourier implements ISignalProcessor {
	
	/**
	 * Private instance of singleton.
	 */
	private static FastFourier instance;

	/**
	 * Processes input signal using Fast Fourier Transfomation.
	 * Handles signal extension with zeros if neccessary.
	 * 
	 * @param signal input signal
	 * @throws IllegalArgumentException
	 * @return result
	 */
	@Override
	public FastFourierResult processSignal(double[] signal) {
		if (signal == null) {
			throw new IllegalArgumentException("Signal must be provided.");
		}
				
		// zero padding to length match power of 2
		int power2 = Mathematic.newMajorNumberOfPowerBase2(signal.length);
		if (power2 != signal.length) {
			signal = Arrays.copyOf(signal, power2);
		}
		
		
		FastFourierTransformer fft = new FastFourierTransformer();
		Complex[] res;
		try {
			res = fft.transform(signal);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e.getMessage());
		} catch (MathException e) {
			throw new RuntimeException(e.getMessage());
		}
		
		return new FastFourierResult(res);
	}
	
	/**
	 * Gets instance of this singleton.
	 * 
	 * @return instance of this singleton
	 */
	public static FastFourier getInstance() {
		if (instance == null) {
			instance = new FastFourier();
		}
		
		return instance;
	}	
}
