package cz.zcu.kiv.eegdsp.fourier.algorithm;

import cz.zcu.kiv.eegdsp.utils.ComplexNumber;

/**
 * Fast Fourier Transformation.
 * 
 * @author Tom ondk
 * 
 * @deprecated In favor of org.apache.commons.math.transform.FastFourierTransformer
 */
public class FFT
{
	/**
	 * Precalculated complex value table used in FFT.
	 */
	private static ComplexNumber[] wtbl = null;
	
	/**
	 * Last number of used values from <code>wtbl</code>.
	 */
	private static int lastValueOfN = 0;
	
	/**
	 * Creates precalculated complex value table used in FFT.
	 * 
	 * @param n numer of table values
	 * @return precalculated complex value table used in FFT
	 */
	private static ComplexNumber[] createWTBL(int n)
	{
		if (lastValueOfN == n)
			return wtbl;
		
		int n2 = n >> 1;
		wtbl = new ComplexNumber[n2 + 1];
		wtbl[0] = new ComplexNumber(1, 0);
		
		double t = 0;
		for (int i = 1; i < n2; i++)
		{
			t = 2 * Math.PI * i / n;
			wtbl[i] = new ComplexNumber(Math.cos(t), Math.sin(t));
		}
		
		wtbl[n2] = new ComplexNumber(-1, 0);
		lastValueOfN = n;
		
		return wtbl;
	}
	/**
	 * Calculates Fourier transformation of input signal using FFT.
	 * 
	 * @deprecated
	 * @param data input signal
	 * @return Fourier transformation of signal
	 */
	public static ComplexNumber[] doFFT(double[] data)
	{
		ComplexNumber[] f = new ComplexNumber[data.length];
		
		for (int i = 0; i < data.length; i++)
		{
			f[i] = new ComplexNumber(data[i], 0);
		}
		
		int n = f.length;
		int n2 = n >> 1;
		ComplexNumber[] wtbl = createWTBL(n);
		
		for (int i = 0, j = 0, k;;)
		{
			if (i < j)
			{
				ComplexNumber t = f[i];
				f[i] = f[j];
				f[j] = t;
			}
			if (++i >= n)
				break;
			
			for (k = n2; k <= j; k >>= 1)
			{
				j -= k;
			}
			j += k;
		}
		ComplexNumber tm = f[0].clone();
		
		int k = 1;
		while (k < n)
		{
			int h = 0;
			int k2 = k << 1;
			int d = n / k2;
			
			for (int j = 0; j < k; j++)
			{
				for (int i = j; i < n; i += k2)
				{
					int ik = i + k;
					f[ik].timesBy(wtbl[n2 - h]);
					tm.set(f[ik]).negate().addTo(f[i]);
					f[ik].addTo(f[i]);
					f[i].set(tm);
				}
				h += d;
			}
			k = k2;
		}
		return f;
	}
}
