package cz.zcu.kiv.eegdsp.main;

import cz.zcu.kiv.eegdsp.fourier.FastFourier;
import cz.zcu.kiv.eegdsp.matchingpursuit.MatchingPursuit;
import cz.zcu.kiv.eegdsp.wavelet.continuous.WaveletTransformationContinuous;
import cz.zcu.kiv.eegdsp.wavelet.discrete.WaveletTransformationDiscrete;

/**
 * Class for signal processing methods initialization.
 * Singleton. 
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 29.12.2010
 *
 */
public class SignalProcessingFactory {
	
	/**
	 * Private instance of this singleton. 
	 */
	private static SignalProcessingFactory instance;
	
	/**
	 * Private constructor of singleton.
	 */
	private SignalProcessingFactory() {
		
	}
	
	/**
	 * Gets execution interface for matching pursuit.
	 * @return matching pursuit execution interface
	 */
	public MatchingPursuit getMatchingPursuit() {
		return MatchingPursuit.getInstance();
	}
	
	/**
	 * Gets execution interface for discrete wavelet transformation.
	 * @return discrete wavelet transformation execution interface
	 */
	public WaveletTransformationDiscrete getWaveletDiscrete() {
		return WaveletTransformationDiscrete.getInstance();
	}
	
	/**
	 * Gets execution interface for continuous wavelet transformation.
	 * @return continuous wavelet transformation execution interface
	 */
	public WaveletTransformationContinuous getWaveletContinuous() {
		return WaveletTransformationContinuous.getInstance();
	}
	
	/**
	 * Gets execution interface for fast fourier transformation.
	 * @return fast fourier transformation execution interface
	 */
	public FastFourier getFastFourier() {
		return FastFourier.getInstance();
	}
	
	/**
	 * Gets singleton instance.
	 * 
	 * @return singleton instance
	 */
	public static SignalProcessingFactory getInstance() {
		if (instance == null) {
			instance = new SignalProcessingFactory();
		}
		
		return instance;
	}
}
