package cz.zcu.kiv.eegdsp.main;

import java.util.Arrays;

import cz.zcu.kiv.eegdsp.common.ISignalProcessingResult;
import cz.zcu.kiv.eegdsp.common.ISignalProcessor;
import cz.zcu.kiv.eegdsp.matchingpursuit.MatchingPursuit;
import cz.zcu.kiv.eegdsp.wavelet.continuous.WaveletTransformationContinuous;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.WaveletCWT;
import cz.zcu.kiv.eegdsp.wavelet.discrete.WaveletResultDiscrete;
import cz.zcu.kiv.eegdsp.wavelet.discrete.WaveletTransformationDiscrete;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.WaveletDWT;

/**
 * Testing class with app entry point.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 29.12.2010
 *
 */
public class Test {
	
	/**
	 * Testing signal.
	 */
	private static final double[] SIGNAL = new double[] {
		1, 2, 5, 10, 5, 7, 9, 1, 3, 5, 4
	};
	
	/**
	 * Tests DWT.
	 */
	public static void testDwt() {
		System.out.println("\tDiscrete wavelet transformation\n");		
		
		ISignalProcessor dwt = SignalProcessingFactory.getInstance().getWaveletDiscrete();
		
		String[] names = ((WaveletTransformationDiscrete)dwt).getWaveletGenerator().getWaveletNames();
		System.out.println(Arrays.toString(names));
		
		for (String name : names) {
			System.out.println("\t" + name + "...");
			
			WaveletDWT wavelet = null;
			try {
				wavelet = ((WaveletTransformationDiscrete)dwt).getWaveletGenerator().getWaveletByName(name);
								
			} catch (Exception e) {
				System.out.println("Exception loading wavelet " + name + ".");
			}
			
			if (wavelet == null) {
				continue;
			}
			
			double[] signal = Arrays.copyOf(SIGNAL, SIGNAL.length);
			
			((WaveletTransformationDiscrete)dwt).setWavelet(wavelet);
			
			ISignalProcessingResult res = dwt.processSignal(signal);
			
			res.toArray();
			res.toHashMap();
			System.out.println(res.toString());
			
			res = ((WaveletTransformationDiscrete)dwt).reconstructSignal(((WaveletResultDiscrete)res).getDwtCoefficients());
			res.toArray();
			res.toHashMap();
			System.out.println(res.toString());
		}
		
		System.out.println("\n*******************************************************\n");
	}
	
	/**
	 * Tests CWT.
	 */
	public static void testCwt() {
		System.out.println("\tContinuous wavelet transformation\n");
		
		ISignalProcessor cwt = SignalProcessingFactory.getInstance().getWaveletContinuous();
		
		String[] names = ((WaveletTransformationContinuous)cwt).getWaveletGenerator().getWaveletNames();
		System.out.println(Arrays.toString(names));	
		
		for (String name : names) {
			System.out.println("\t" + name + "...");
			
			WaveletCWT wavelet = null;
			try {
				wavelet = ((WaveletTransformationContinuous)cwt).getWaveletGenerator().getWaveletByName(name);
								
			} catch (Exception e) {
				System.out.println("Exception loading wavelet " + name + ".");
			}
			
			if (wavelet == null) {
				continue;
			}
			
			double[] signal = Arrays.copyOf(SIGNAL, SIGNAL.length);
			
			((WaveletTransformationContinuous)cwt).setWavelet(wavelet);
			
			ISignalProcessingResult res = cwt.processSignal(signal);
			
			res.toArray();
			res.toHashMap();
			System.out.println(res.toString());
		}
		
		System.out.println("\n*******************************************************\n");
	}

	/**
	 * Tests MP.
	 */
	public static void testMP() {
		System.out.println("\tMatching pursuit\n");
		
		double[] signal = Arrays.copyOf(SIGNAL, SIGNAL.length);
		
		ISignalProcessor mp = SignalProcessingFactory.getInstance().getMatchingPursuit();
		((MatchingPursuit)mp).setIterationCount(10);
		ISignalProcessingResult res = mp.processSignal(signal);
		
		res.toArray();
		res.toHashMap();
		System.out.println(res.toString());
		
		System.out.println("\n*******************************************************\n");
	}
	
	/**
	 * Tests FFT.
	 */
	public static void testFFT() {
		System.out.println("\tFast Fourier Transformation\n");
		
		double[] signal = Arrays.copyOf(SIGNAL, SIGNAL.length);
		
		ISignalProcessor fft = SignalProcessingFactory.getInstance().getFastFourier();
		ISignalProcessingResult res = fft.processSignal(signal);
		
		res.toArray();
		res.toHashMap();
		System.out.println(res.toString());
		
		System.out.println("\n*******************************************************\n");
	}
	
	/**
	 * Testing application entry point.
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("Testing module for Signal processing library 1.0");
		System.out.println("\n*******************************************************\n");
				
		Test.testDwt();
		Test.testCwt();
		Test.testMP();
		Test.testFFT();
		
		System.out.println("\tTests done.");
	}
}
