package cz.zcu.kiv.eegdsp.matchingpursuit.algorithm;
/**
 * Abstract class of atom used in matching pursuit decomposition.
 * 
 * @author Tom ondk
 */
public abstract class Atom
{
	/**
	 * Scale of atom.
	 */
	protected double scale;
	

	/** 
	 * Gets atom scale.
	 * @return atom scale.
	 */
	public double getScale()
	{
		return scale;
	}
	
	/**
	 * Sets atom scale.
	 * @param scale atom scale
	 */
	public void setScale(double scale)
	{
		this.scale = scale;
	}
	
	/**
	 * Position of atom.
	 */
	protected double position;
	
	/**
	 * Gets position of atom.
	 * 
	 * @return position of atom
	 */
	public double getPosition()
	{
		return position;
	}
	
	/**
	 * Gets position of atom.
	 * 
	 * @param position position of atom
	 */
	public void setPosition(double position)
	{
		this.position = position;
	}
	
	/**
	 * Coefficient of Gabor atom similarity to input signal.
	 */
	protected double difference;
	
	/**
	 * Gets coefficient of Gabor atom similarity to input signal.
	 * 
	 * @return coefficient of Gabor atom similarity to input signal
	 */
	public double getDifference()
	{
		return difference;
	}
	
	/**
	 * Sets coefficient of Gabor atom similarity to input signal.
	 * 
	 * @param difference coefficient of Gabor atom similarity to input signal
	 */
	public void setDifference(double difference)
	{
		this.difference = difference;
	}
	
	/**
	 * Gets function values stretched to length.
	 * @param length requested length
	 * @return function values
	 */
	public abstract double[] getValues(int length);
	
	/**
	 * Subtracts atom from input signal.
	 * @param signal signal
	 */
	public abstract void subtractFrom(double[] signal);
	
	/**
	 * Gets function value of sample on index.
	 * @param numberOfSample index of sample
	 * @return function value of sample
	 */
	public abstract double evaluate(double numberOfSample);
}
