package cz.zcu.kiv.eegdsp.matchingpursuit.algorithm;

import java.util.Arrays;
/**
 * User defined atom.
 * 
 * @author Tom ondk
 */
public class UsersAtom extends Atom
{
	/*
	 * Attributes.
	 */
	
	/**
	 * Name.
	 */
	protected String name;

	/**
	 * Vrac hodnotu parametru <code>name</code>.
	 * @return Pojmenovn atomu.
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 * Sets name.
	 * @param name name
	 */
	public void setName(String name)
	{
		if (name == null) {
			name = "";
		}
		
		this.name = name;
	}
	
	/**
	 * Atom stretch.
	 */
	protected double stretch;
	/**
	 * Gets stretch.
	 * @return stretch
	 */
	public double getStretch()
	{
		return stretch;
	}
	/**
	 * Sets stretch.
	 * @param stretch stretch
	 */
	public void setStretch(double stretch)
	{
		this.stretch = stretch;
	}
	/**
	 * Function values.
	 */
	protected double[] values;
	
	/**
	 * Sets reference to function value array.
	 * @param values reference to function value array
	 */
	public void setValues(double[] values)
	{
		this.values = values;
	}
	
	/**
	 * Gets function values stretched to length.
	 * @param length of values
	 */
	public double[] getValues(int length)
	{
		length = Math.min(this.values.length, length);
		double[] retval = Arrays.copyOf(this.values, length);
		
		return retval;
	}
	
	/*
	 * Constructors.
	 */
	
	/**
	 * Empty constructor.
	 */
	public UsersAtom() {
		this("");
	}
	
	/**
	 * Constructor of atom with name.
	 * 
	 * @param name name for atom
	 */
	public UsersAtom(String name) {
		this.setName(name);
	}
	
	/*
	 * Methods.
	 */
	
	/**
	 * Evaluates atom in specified time.
	 * @param numberOfSample time
	 * @return function value in time
	 */
	@Override
	public double evaluate(double numberOfSample)
	{
		if (numberOfSample >= this.values.length) {
			throw new IllegalArgumentException("Out of values range.");
		}
		int t = (int) numberOfSample;
		return values[t];
	}
	/**
	 * Subtracts atom from signal
	 * @param signal signal (input/output parameter)
	 */
	@Override
	public void subtractFrom(double[] signal)
	{
		for (int i = 0; i < signal.length; i++)
		{
			signal[i] -= values[i];
		}
	}
	/**
	 * Gets text representation.
	 * @return string representation
	 */
	@Override
	public String toString()
	{
		return name + " - position: " + position + ", scale: " + scale + ", stretch: " + stretch + ", difference: " + difference;
	}
}
