package cz.zcu.kiv.eegdsp.utils;

/**
 * Complex number representation.
 *
 * @author Tom ondk
 * 
 * @deprecated In favour of org.apache.commons.math.complex.Complex
 */
public class ComplexNumber
{
	/**
	 * Ignored difference from zero. Values (<code>0 +- ZERO_ALLOWANCE</code>) considered equal to zero.
	 */
	public static final double ZERO_ALLOWANCE = 1.0e-7F;
	
	/**
	 * Real part.
	 */
	private double re;

	/**
	 * Gets real part.
	 * @return real part
	 */
	public double getRe()
	{
		return re;
	}
	
	/**
	 * Sets real part.
	 * 
	 * @param re real part
	 */
	public void setRe(double re) {
		this.re = re;
	}
	
	/**
	 * Imaginary part.
	 */
	private double im;

	/**
	 * Gets imaginary part.
	 * @return imaginary part
	 */
	public double getIm()
	{
		return im;
	}
	
	/**
	 * Sets imaginary part.
	 * 
	 * @param im imaginary part
	 */
	public void setIm(double im) {
		this.im = im;
	}
	
	/**
	 * Creates complex zero.
	 */
	public ComplexNumber() {
		this(0, 0);
	}
	
	/**
	 * Creates complex number clone.
	 */
	public ComplexNumber(ComplexNumber pattern) {
		this(pattern.re, pattern.im);
	}
	
	/**
	 * Creates complex number.
	 * 
	 * @param re real part
	 * @param im imaginary part
	 */
	public ComplexNumber(double re, double im)
	{
		this.re = re;
		this.im = im;
	}
	/**
	 * Sets this instance to match specified complex number.
	 * 
	 * @param complex pattern to use
	 * @return this instance after change
	 */
	public ComplexNumber set(ComplexNumber complex)
	{
		this.re = complex.re;
		this.im = complex.im;
		return this;
	}
	/**
	 * Zero test using <code>ZERO_ALLOWANCE</code>.
	 * 
	 * @return <b>true</b>, if zero, <b>false</b> otherwise.
	 */
	public boolean isZero()
	{
		return (Math.abs(this.re) < ZERO_ALLOWANCE && Math.abs(this.im) < ZERO_ALLOWANCE);
	}
	
	/**
	 * Adds complex number to this instance.
	 * 
	 * @param complex added operand
	 * @return result (this instance)
	 */
	public ComplexNumber addTo(ComplexNumber complex)
	{
		this.re += complex.re;
		this.im += complex.im;
		return this;
	}
	
	/**
	 * Multiplies this instance by complex number.
	 * 
	 * @param complex multiplicator
	 * @return result (this instance)
	 */
	public ComplexNumber timesBy(ComplexNumber complex)
	{
		double tmp = this.re * complex.re - this.im * complex.im;
		this.im = this.im * complex.re + this.re * complex.im;
		this.re = (double) tmp;
		return this;
	}
	
	/**
	 * Negates this instance.
	 * 
	 * @return result (this instance)
	 */
	public ComplexNumber negate()
	{
		this.re = -this.re;
		this.im = -this.im;
		return this;
	}
	
	/**
	 * Divides this instance by real divider.
	 * 
	 * @param n divider
	 * @return result (this instance)
	 */
	public ComplexNumber div(double n)
	{
		this.re /= n;
		this.im /= n;
		return this;
	}
		
	/**
	 * Gets size of this instance.
	 * 
	 * @return size
	 */
	public double getSize() {
		return Math.sqrt(this.re * this.re + this.im * this.im);
	}
	
	/**
	 * Gets angle of 2D representation from positive x axis.
	 * 
	 * @return angle in radians
	 */
	public double getAngle() {
		return Math.atan2(this.im, this.re);
	}
	
	/**
	 * Creates clone of this instance.
	 * @return clone
	 */
	@Override
	public ComplexNumber clone()
	{
		return new ComplexNumber(this);
	}
	/**
	 * Gets text representatiton.
	 * @return string for this instance
	 */
	@Override
	public String toString()
	{
		if (im >= 0)
		{
			return re + " + " + Math.abs(im) + "i";
		}
		else
		{
			return re + " - " + Math.abs(im) + "i";
		}
	}
}
