package cz.zcu.kiv.eegdsp.wavelet.continuous;

import cz.zcu.kiv.eegdsp.wavelet.common.WaveletGenerator;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.ComplexGaussian;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.ComplexMorlet;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.Gaussian;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.MexicanHat;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.Morlet;
import cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets.WaveletCWT;

/**
 * Generator of continuous wavelets. Singleton.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class WaveletGeneratorContinuous extends WaveletGenerator {
	
	/**
	 * Private instance of singleton.
	 */
	private static WaveletGeneratorContinuous instance;
	
	/**
	 * Available wavelet classes.
	 */
	private static final Class<?>[] wavelets = new Class[] {
		ComplexGaussian.class,
		ComplexMorlet.class,
		Gaussian.class,
		MexicanHat.class,
		Morlet.class
	};
	
	/**
	 * Private constructor.
	 */
	private WaveletGeneratorContinuous() {
		
	}
	
	/**
	 * Gets intance of wavelet by its name.
	 * 
	 * @throws RuntimeException
	 * @throws IllegalArgumentException
	 * 
	 * @return intance of wavelet by its name
	 */
	@Override
	public WaveletCWT getWaveletByName(String name) {
		if (name == null || name.length() == 0) {
			throw new IllegalArgumentException("Wavelet name must be provided.");
		}
		
		try {
			Object rawWavelet = this.getWaveletByName(name, WaveletGeneratorContinuous.wavelets);
			
			if (rawWavelet == null || !(rawWavelet instanceof WaveletCWT)) {
				throw new IllegalArgumentException("Wavelet with specified name not found.");
			}
			
			return (WaveletCWT)rawWavelet;
			
		} catch (Exception e) {
			throw new RuntimeException("Couldn't intialize specified wavelet.");
		}
	}
	
	/**
	 * Gets classes of wavelets available.
	 * 
	 * @return classes of wavelets available
	 */
	@Override
	protected Class<?>[] getWaveletClasses() {
		return WaveletGeneratorContinuous.wavelets;
	}
	
	/**
	 * Gets instance of this singleton.
	 * @return singleton instance
	 */
	public static WaveletGeneratorContinuous getInstance() {
		if (instance == null) {
			instance = new WaveletGeneratorContinuous();
		}
		
		return instance;
	}
}
