package cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets;

/**
 * ComplexMorlet wavelet.
 */
public class ComplexMorlet extends WaveletCWT
{
	/**
	 * Name of wavelet.
	 */
	private final static String NAME = "Complex_Morlet";
	
	/**
	 * Main legth of wavelet.
	 */
	private final static int MAIN_LENGTH = 16;
	
	/**
	 * Frequence bandwidth constant.
	 */
	private double fb;
	
	/**
	 * Frequence center constant.
	 */
	private double fc;
	
	/**
	 * Norming constant.
	 */
	private double norm;
	
	/**
	 * Wavelet contructor with default parameters (1, 1). 
	 */
	public ComplexMorlet() {
		this(1.0, 1.0);
	}
	
	/**
	 * Wavelet contructor.
	 * 
	 * @param fb
	 * @param fc
	 */
	public ComplexMorlet(double fb, double fc)
	{
		super(NAME, MAIN_LENGTH);
		this.setFB(fb);
		this.setFC(fc);
		this.norm = 1.0 / Math.sqrt(Math.PI * fb);	
	}
	
	/**
     * Real part of mother wavelet.
     * 
     * @param t time
     * @param a dilatation (ignored)
     * @return real part of mother wavelet
     */
	@Override
	public double reCoef(double t, double a)
	{
		double tPow2 = t*t;
		
		return norm * Math.cos(2.0 * Math.PI * fc * t) * Math.exp(-tPow2/fb); 
	}
	
	/**
     * Imaginary part of mother wavelet.
     * 
     * @param t time
     * @param a dilatation (ignored)
     * @return imaginary part of mother wavelet
     */
	@Override
	public double imCoef(double t, double a)
	{
		double tPow2 = t*t;
		
		return norm * Math.sin(2.0 * Math.PI * fc * t) * Math.exp(-tPow2/fb); 
	}
	
	/**
	 * Sets frequence bandwidth constant.
	 */
	public void setFB(double fb)
	{
		this.fb = fb;
		this.norm = 1.0 / Math.sqrt(Math.PI * fb);
	}
	
	/**
	 * Sets frequence center constant.
	 */
	public void setFC(double fc)
	{
		this.fc = fc;
	}
}
