package cz.zcu.kiv.eegdsp.wavelet.continuous.algorithm.wavelets;

/**
 * Abstract continuous wavelet.
 */
public abstract class WaveletCWT
{
	/**
	 * Wavelet name.
	 */
	private String name;
	
	/**
	 * Main legth of wavelet.
	 */
	private int mainLength;

	/**
	 * Constructor for wavelet.
	 *  
	 * @param name name of wavelet
	 * @param mainLength main length of wavelet
	 */
    public WaveletCWT(String name, int mainLength)
    {
    	this.name = name;
    	this.mainLength = mainLength;
    }

    /**
     * Real part of mother wavelet.
     * 
     * @param t
     * @param a
     * @return real part of mother wavelet
     */
    public abstract double reCoef(double t, double a);
    
    /**
     * Imaginary part of mother wavelet.
     * 
     * @param t
     * @param a
     * @return imaginary part of mother wavelet
     */
    public abstract double imCoef(double t, double a);
    
    /**
     * Gets wavelet name.
     *  
     * @return wavelet name
     */
    public String getName()
    {
        return name;
    }
    
    /**
     * Gets main length of wavelet.
     * 
     * @return main length of wavelet
     */
    public int getMainLength()
    {
        return mainLength;
    }
}
