package cz.zcu.kiv.eegdsp.wavelet.discrete;

import cz.zcu.kiv.eegdsp.wavelet.common.WaveletGenerator;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Coiflet12;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Coiflet18;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Coiflet24;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Coiflet30;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Coiflet6;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies10;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies12;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies14;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies16;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies18;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies20;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies4;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies6;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Daubechies8;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Haar;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Symmlet4;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Symmlet6;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.Symmlet8;
import cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets.WaveletDWT;

/**
 * Generator of discrete wavelets.
 * 
 * @author Petr Kellnhofer, A10N0018P
 * @date 2.1.2011
 *
 */
public class WaveletGeneratorDiscrete extends WaveletGenerator {
	
	/**
	 * Private instance of singleton.
	 */
	private static WaveletGeneratorDiscrete instance;
		
	/**
	 * Available wavelet classes.
	 */
	private static final Class<?>[] wavelets = new Class[] {
		Coiflet6.class,
		Coiflet12.class,
		Coiflet18.class,
		Coiflet24.class,
		Coiflet30.class,
		Daubechies4.class,
		Daubechies6.class,
		Daubechies8.class,
		Daubechies10.class,
		Daubechies12.class,
		Daubechies14.class,
		Daubechies16.class,
		Daubechies18.class,
		Daubechies20.class,
		Haar.class,
		Symmlet4.class,
		Symmlet6.class,
		Symmlet8.class
	};
	
	/**
	 * Private constructor.
	 */
	private WaveletGeneratorDiscrete() {
		
	}
	
	/**
	 * Gets intance of wavelet by its name.
	 * 
	 * @throws RuntimeException
	 * @throws IllegalArgumentException
	 * 
	 * @return intance of wavelet by its name
	 */
	@Override
	public WaveletDWT getWaveletByName(String name) {
		if (name == null || name.length() == 0) {
			throw new IllegalArgumentException("Wavelet name must be provided.");
		}
		
		try {
			Object rawWavelet = this.getWaveletByName(name, WaveletGeneratorDiscrete.wavelets);
			
			if (rawWavelet == null || !(rawWavelet instanceof WaveletDWT)) {
				throw new IllegalArgumentException("Wavelet with specified name not found.");
			}
			
			return (WaveletDWT)rawWavelet;
			
		} catch (Exception e) {
			throw new RuntimeException("Couldn't intialize specified wavelet.");
		}
	}
	
	/**
	 * Gets classes of wavelets available.
	 * 
	 * @return classes of wavelets available
	 */
	@Override
	protected Class<?>[] getWaveletClasses() {
		return WaveletGeneratorDiscrete.wavelets;
	}
	
	/**
	 * Gets instance of this singleton.
	 * @return singleton instance
	 */
	public static WaveletGeneratorDiscrete getInstance() {
		if (instance == null) {
			instance = new WaveletGeneratorDiscrete();
		}
		
		return instance;
	}
}
