package cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets;

/**
 * Class of Daubechies4 discrete wavelet.
 * Contains all necessary coefficients for direct and inverse transformation.
 */
public class Daubechies4 extends WaveletDWT
{
	/**
	 * Name of wavelet.
	 */
	private final static String NAME = "Daubechies_4";
	
	/**
	 * Square of three constant.
	 */
	private final static double SQRT_3 = Math.sqrt(3);
	
	/**
	 * Denormalizer constant.
	 */
	private final static double DENOM = 4 * Math.sqrt(2);
	
	/**
	 * Scale coefficients.
	 */
	private final static double[] SCALE = 	{	(1 + SQRT_3) / DENOM,
												(3 + SQRT_3) / DENOM,
												(3 - SQRT_3) / DENOM,
												(1 - SQRT_3) / DENOM};
	
	/**
	 * Wavelet coefficients.
	 */
	private final static double[] WAVELET = {	-SCALE[3], SCALE[2],
												-SCALE[1], SCALE[0]	};
	
	
	/**
	 * Scale coefficients for inverse transformation.
	 */
	private final static double[] I_SCALE = 	{	SCALE[2], WAVELET[2],
													SCALE[0], WAVELET[0]};
	
	/**
	 * Wavelet coefficients for inverse transformation.
	 */
	private final static double[] I_WAVELET = 	{	SCALE[3], WAVELET[3],
													SCALE[1], WAVELET[1]};	
	
	/**
	 * Wavelet Daubechies4 constructor.
	 */
	public Daubechies4()
	{
		super(NAME, SCALE, WAVELET, I_SCALE, I_WAVELET);
	}
}
