package cz.zcu.kiv.eegdsp.wavelet.discrete.algorithm.wavelets;

import cz.zcu.kiv.eegdsp.utils.Mathematic;

/**
 * Abstract class of discrete wavelet transformation.
 */
public abstract class WaveletDWT
{
	/**
	 * Name of wavelet.
	 */
	private String name;
	

    /**
     * Gets name of wavelet.
     * 
     * @return name of wavelet
    */
    public String getName()
    {
        return name;
    }
	
	/**
	 * Scale coefficients.
	 */
	private double[] scale;
	
	/**
     * Gets scale coefficients.
     * 
	 * @return array of scale coefficients
	 */
	public double[] getScaleArray()
	{
		return scale;
	}
	
	/**
     * Gets scale coefficient at index i.
     * 
	 * @return scale coefficient at index i
	 */
	public double getScaleCoefx(int i)
	{
		return scale[i];
	}
	
	/**
	 * Wavelet coefficients.
	 */
	private double[] wavelet;
	
	/**
     * Gets wavelet coefficients.
     * 
	 * @return wavelet coefficients.
	 */
	public double[] getWaveletArray()
	{
		return wavelet;
	}
	
	/**
     * Gets wavelet coefficient at index i.
     * 
	 * @return wavelet coefficient at index i
	 */
	public double getWaveletCoefx(int i)
	{
		return wavelet[i];
	}
	
	/**
	 * Scale coefficients for inverse transformation.
	 */
	private double[] iScale;
	
	/**
     * Gets scale coefficients for inverse transformation.
     * 
	 * @return scale coefficients for inverse transformation.
	 */
	public double[] getIScaleArray()
	{
		return iScale;
	}
	
	/**
     * Gets scale coefficient at index i for inverse transformation.
     * 
	 * @return scale coefficient at index i for inverse transformation.
	 */
	public double getIScaleCoefx(int i)
	{
		return iScale[i];
	}
	
	/**
	 * Wavelet coefficients for inverse transformation.
	 */
	private double[] iWavelet;
	
	/**
     * Gets wavelet coefficients for inverse transformation
     * 
	 * @return wavelet coefficients for inverse transformation
	 */
	public double[] getIWaveletArray()
	{
		return iWavelet;
	}
    
    /**
     * Gets wavelet coefficient at index i for inverse transformation.
     * 
	 * @return wavelet coefficient at index i for inverse transformation
	 */
    public double getIWaveletCoefx(int i)
    {
    	return iWavelet[i];
    }    
    	
	
	/**
	 * Discrete wavelet constructor.
	 * 
	 * @param name name of wavelet
	 * @param scale scale coefficients
	 * @param wavelet wavelet coefficients
	 * @param iScale scale coefficients for inverse transformation
	 * @param iWavelet wavelet coefficients for inverse transformation
	 */
	public WaveletDWT(String name, double[] scale, double[] wavelet, double[] iScale, double[] iWavelet)
	{
		if (name == null || scale == null || wavelet == null || iScale == null || iWavelet == null) {
			throw new IllegalArgumentException("No null arguments allowed");
		}
		
		this.name = name;
		this.scale = scale;
		this.wavelet = wavelet;
		this.iScale = iScale;
		this.iWavelet = iWavelet;
	}
	
	/**
	 * Gets wavelet length.
	 * 
	 * @return length of wavelet
	 */
	public int getLength() {
		return this.scale.length;
	}
	
	/**
	 * Gets nearest bigger power of two searched from wavelet length up.
	 * 
	 * @return nearest bigger power of two searched from wavelet length up
	 */
	public int getMajorWaveletLengthOfPow2() {
		return Mathematic.newMajorNumberOfPowerBase2(this.getLength());
	}
}
