/*******************************************************************************
 * This file is part of the EEG-database project
 * 
 *   ==========================================
 *  
 *   Copyright (C) 2013 by University of West Bohemia (http://www.zcu.cz/en/)
 *  
 *  ***********************************************************************************************************************
 *  
 *   Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *   the License. You may obtain a copy of the License at
 *  
 *       http://www.apache.org/licenses/LICENSE-2.0
 *  
 *   Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 *   an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 *   specific language governing permissions and limitations under the License.
 *  
 *  ***********************************************************************************************************************
 *  
 *   ArticleComment.java, 2013/10/02 00:01 Jakub Rinkes
 ******************************************************************************/
package cz.zcu.kiv.eegdatabase.data.pojo;

import cz.zcu.kiv.eegdatabase.data.annotation.SolrField;
import cz.zcu.kiv.eegdatabase.data.annotation.SolrId;
import cz.zcu.kiv.eegdatabase.logic.indexing.IndexField;

import javax.persistence.*;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Jiri Vlasimsky
 */
@Entity
//@Indexed//Mark for indexing
//@Analyzer(impl = StandardAnalyzer.class)
@javax.persistence.Table(name = "ARTICLES_COMMENTS")
public class ArticleComment implements Serializable {

    private static final long serialVersionUID = -8594589377952874454L;
    
    //@DocumentId
    @SolrId
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "COMMENT_ID")
    private int commentId;
    @ManyToOne
    @JoinColumn(name = "PERSON_ID")
    private Person person;
    @ManyToOne
    @JoinColumn(name = "PARENT_ID")
    private ArticleComment parent;
    @OneToMany(mappedBy = "parent")
    private Set<ArticleComment> children = new HashSet<ArticleComment>(0);
    /*
    @Fields({
            @Field(index = Index.TOKENIZED), // same property indexed multiple times
            @Field(store = Store.YES), // text value is stored in the index
            @Field(name = "text")})
    */
    @SolrField(name = IndexField.TEXT)
    @Column(name = "TEXT")
    @Lob
    private String text;
    @Column(name = "TIME")
    private Timestamp time;
    @ManyToOne
    @JoinColumn(name = "ARTICLE_ID")
    private Article article;
    private boolean userMemberOfGroup; // changes dynamically from app
    private boolean userIsOwnerOrAdmin; // changes dynamically from app

    public ArticleComment() {
    }

    public int getCommentId() {
        return commentId;
    }

    public void setCommentId(int commentId) {
        this.commentId = commentId;
    }

    public ArticleComment getParent() {
        return parent;
    }

    public void setParent(ArticleComment parent) {
        this.parent = parent;
    }

    public Person getPerson() {
        return person;
    }

    public void setPerson(Person person) {
        this.person = person;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public Timestamp getTime() {
        return time;
    }

    public void setTime(Timestamp time) {
        this.time = time;
    }

    public boolean isUserIsOwnerOrAdmin() {
        return userIsOwnerOrAdmin;
    }

    public void setUserIsOwnerOrAdmin(boolean userIsOwnerOrAdmin) {
        this.userIsOwnerOrAdmin = userIsOwnerOrAdmin;
    }

    public boolean isUserMemberOfGroup() {
        return userMemberOfGroup;
    }

    public void setUserMemberOfGroup(boolean userMemberOfGroup) {
        this.userMemberOfGroup = userMemberOfGroup;
    }

    public Article getArticle() {
        return article;
    }

    public void setArticle(Article article) {
        this.article = article;
    }

    public Set<ArticleComment> getChildren() {
        return children;
    }

    public void setChildren(Set<ArticleComment> children) {
        this.children = children;
    }
}
